"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkdownThemeConverterContext = void 0;
const typedoc_1 = require("typedoc");
const utils_1 = require("./support/utils");
class MarkdownThemeConverterContext {
    constructor(theme, options) {
        this.theme = theme;
        this.options = options;
        this._anchors = {};
        this.readmeTemplate = (pageEvent) => {
            return this.theme.getRenderContext().templates.readmeTemplate(pageEvent);
        };
        this.projectTemplate = (pageEvent) => {
            return this.theme.getRenderContext().templates.projectTemplate(pageEvent);
        };
        this.reflectionTemplate = (pageEvent) => {
            return this.theme
                .getRenderContext()
                .templates.reflectionTemplate(pageEvent);
        };
        this.memberTemplate = (pageEvent) => {
            return this.theme.getRenderContext().templates.memberTemplate(pageEvent);
        };
    }
    getOption(name) {
        return this.options.getValue(name);
    }
    getUrls(project) {
        var _a;
        if (!this._urls) {
            const readme = this.getOption('readme');
            const entryDocument = this.getOption('entryDocument');
            const noReadmeFile = readme.endsWith('none');
            const modulesFile = 'modules.md';
            this._urls = [];
            if (noReadmeFile) {
                project.url = entryDocument;
                this._urls.push(new typedoc_1.UrlMapping(entryDocument, project, this.projectTemplate));
            }
            else {
                project.url = modulesFile;
                this._urls.push(new typedoc_1.UrlMapping(entryDocument, project, this.readmeTemplate));
                this._urls.push(new typedoc_1.UrlMapping(`${this.getOption('numberPrefixOutput') ? '01-' : ''}${modulesFile}`, project, this.projectTemplate));
            }
            (_a = project.groups) === null || _a === void 0 ? void 0 : _a.forEach((projectGroup, groupIndex) => {
                projectGroup.children.forEach((child) => {
                    const index = noReadmeFile ? groupIndex : groupIndex + 1;
                    this.buildUrls(child, index, projectGroup);
                });
            });
        }
        return this._urls;
    }
    getNavigation(project) {
        var _a;
        const urls = this.getUrls(project);
        const navigation = [];
        const onlyModules = (_a = project.children) === null || _a === void 0 ? void 0 : _a.every((child) => child.kindOf(typedoc_1.ReflectionKind.Module));
        if (onlyModules) {
            urls
                .filter((urlMapping) => {
                var _a;
                return ((_a = urlMapping.model) === null || _a === void 0 ? void 0 : _a.kindOf) &&
                    urlMapping.model.kindOf(typedoc_1.ReflectionKind.Module);
            })
                .forEach((urlMapping) => {
                var _a;
                const children = this.getNavigationGroups(urls, urlMapping.url);
                navigation.push({
                    title: (_a = urlMapping.model) === null || _a === void 0 ? void 0 : _a.name,
                    url: urlMapping.url,
                    ...((children === null || children === void 0 ? void 0 : children.length) && { children }),
                });
            });
        }
        else {
            this.getNavigationGroups(urls).forEach((group) => {
                navigation.push(group);
            });
        }
        return navigation;
    }
    getNavigationGroups(urls, url = '') {
        const groups = urls === null || urls === void 0 ? void 0 : urls.filter((urlMapping) => {
            const baseParts = url === null || url === void 0 ? void 0 : url.split('/').slice(0, -1);
            const urlParts = urlMapping.url.split('/');
            return (urlMapping.model instanceof typedoc_1.ReflectionGroup &&
                (urlParts === null || urlParts === void 0 ? void 0 : urlParts.length) === baseParts.length + 1 &&
                urlMapping.url.startsWith(baseParts.join('/')));
        });
        return groups === null || groups === void 0 ? void 0 : groups.map((group) => {
            var _a;
            return ({
                title: (_a = group.model) === null || _a === void 0 ? void 0 : _a.title,
                children: group === null || group === void 0 ? void 0 : group.model.children.map((child) => {
                    const children = this.getNavigationGroups(urls, child.url);
                    return {
                        title: child.name,
                        url: child.url || null,
                        ...((children === null || children === void 0 ? void 0 : children.length) && { children }),
                    };
                }),
            });
        });
    }
    buildUrls(reflection, index, reflectionGroup) {
        var _a;
        const mapping = this.mappings[reflection.kind];
        if (mapping) {
            const url = this.getUrl(reflection, mapping, index);
            if (mapping.directory) {
                if (!reflection.kindOf(typedoc_1.ReflectionKind.Module)) {
                    const sliceLength = reflection.kindOf(typedoc_1.ReflectionKind.Namespace)
                        ? -2
                        : -1;
                    const groupDirectory = url.split('/').slice(0, sliceLength).join('/');
                    if (this._urls.findIndex((urlMapping) => urlMapping.url === groupDirectory) === -1 &&
                        groupDirectory.endsWith(mapping.directory)) {
                        this._urls.push(new typedoc_1.UrlMapping(groupDirectory, reflectionGroup, () => ''));
                    }
                }
            }
            this._urls.push(new typedoc_1.UrlMapping(url, reflection, mapping.template));
            reflection.url = url;
            reflection.hasOwnDocument = true;
            (_a = reflection.groups) === null || _a === void 0 ? void 0 : _a.forEach((reflectionGroup, groupIndex) => {
                reflectionGroup.children.forEach((child) => {
                    if (mapping.isLeaf) {
                        this.applyAnchorUrl(child, reflection);
                    }
                    else {
                        this.buildUrls(child, groupIndex, reflectionGroup);
                    }
                });
            });
        }
        else if (reflection.parent) {
            this.applyAnchorUrl(reflection, reflection.parent);
        }
    }
    getUrl(reflection, mapping, index) {
        var _a;
        const alias = reflection.getFriendlyFullName().replace(/\//g, '_');
        if (this.getOption('flattenOutputFiles')) {
            return alias + '.md';
        }
        const childrenIncludeNamespaces = this.childrenIncludeNamespaces(reflection);
        const isModuleOrNamespace = reflection.kindOf([
            typedoc_1.ReflectionKind.Module,
            typedoc_1.ReflectionKind.Namespace,
        ]);
        const parts = alias.split('.');
        const namespaces = this.getNamespaces(reflection);
        if (mapping.directory &&
            !isModuleOrNamespace &&
            this.options.getValue('groupByKinds')) {
            parts.splice(parts.length - 1, 0, `${this.getOption('numberPrefixOutput') ? `0${index + 1}-` : ''}${mapping.directory}`);
        }
        if ((this.getOption('kindsWithOwnFile')[0].toLowerCase() !== 'none' &&
            isModuleOrNamespace) ||
            childrenIncludeNamespaces) {
            parts.push(`${parts[parts.length - 1]}`);
        }
        if (namespaces.length > 0) {
            namespaces.forEach((namespaceName) => {
                const namespaceIndex = parts.findIndex((part) => part === namespaceName);
                parts[namespaceIndex] = `${this.getAliasPrefix(typedoc_1.ReflectionKind.Namespace)}.${parts[namespaceIndex]}`;
                parts.splice(namespaceIndex, 0, `${this.getOption('numberPrefixOutput') ? '01-' : ''}${this.mappings[typedoc_1.ReflectionKind.Namespace].directory}`);
            });
        }
        parts[parts.length - 1] = `${this.getAliasPrefix(reflection.kind)}.${parts[parts.length - 1]}`;
        if (parts.length > 1 && ((_a = this.getOption('entryPoints')) === null || _a === void 0 ? void 0 : _a.length) > 1) {
            parts[0] = `${this.getAliasPrefix(typedoc_1.ReflectionKind.Module)}.${parts[0]}`;
        }
        return parts.join('/') + '.md';
    }
    childrenIncludeNamespaces(reflection) {
        var _a;
        return (_a = reflection.children) === null || _a === void 0 ? void 0 : _a.some((child) => child.kindOf(typedoc_1.ReflectionKind.Namespace));
    }
    getAliasPrefix(reflectionKind) {
        return (0, utils_1.slugify)(typedoc_1.ReflectionKind.singularString(reflectionKind));
    }
    getNamespaces(reflection, namespaces = []) {
        var _a;
        if (reflection === null || reflection === void 0 ? void 0 : reflection.kindOf(typedoc_1.ReflectionKind.Namespace)) {
            namespaces.push(reflection.name);
        }
        if ((_a = reflection.parent) === null || _a === void 0 ? void 0 : _a.kindOf(typedoc_1.ReflectionKind.Namespace)) {
            this.getNamespaces(reflection === null || reflection === void 0 ? void 0 : reflection.parent, namespaces);
        }
        return namespaces;
    }
    applyAnchorUrl(reflection, container) {
        var _a, _b;
        if (container.url && !reflection.url) {
            if (!reflection.kindOf(typedoc_1.ReflectionKind.TypeLiteral)) {
                const anchorId = this.getAnchorId(reflection);
                const count = (_b = (_a = this._anchors[container.url]) === null || _a === void 0 ? void 0 : _a.filter((id) => id === anchorId)) === null || _b === void 0 ? void 0 : _b.length;
                const anchor = anchorId + (count > 1 ? '-' + (count - 1).toString() : '');
                reflection.url = container.url + '#' + anchor;
                reflection.anchor = anchor;
            }
            reflection.hasOwnDocument = false;
        }
        reflection.traverse((child) => {
            if (child instanceof typedoc_1.DeclarationReflection) {
                this.applyAnchorUrl(child, container);
            }
        });
    }
    getAnchorId(reflection) {
        const anchorFormat = this.getOption('anchorFormat');
        if (anchorFormat.toLowerCase() === 'lowercase') {
            return reflection.name.toLowerCase();
        }
        if (anchorFormat.toLowerCase() === 'slug') {
            return (0, utils_1.slugify)(reflection.name);
        }
        return reflection.name;
    }
    get mappings() {
        const kindsWithOwnFileOption = this.options.getValue('kindsWithOwnFile');
        const kindsWithOwnFile = Array.isArray(kindsWithOwnFileOption)
            ? kindsWithOwnFileOption.map((val) => val.toLowerCase())
            : [kindsWithOwnFileOption.toLowerCase()];
        const isAll = kindsWithOwnFile.includes('all');
        const mappings = {
            [typedoc_1.ReflectionKind.Module]: {
                isLeaf: false,
                template: this.reflectionTemplate,
                directory: null,
                kind: typedoc_1.ReflectionKind.Module,
            },
            [typedoc_1.ReflectionKind.Namespace]: {
                isLeaf: false,
                template: this.reflectionTemplate,
                directory: 'namespaces',
                kind: typedoc_1.ReflectionKind.Namespace,
            },
        };
        if (isAll || kindsWithOwnFile.includes('class')) {
            mappings[typedoc_1.ReflectionKind.Class] = {
                isLeaf: false,
                template: this.reflectionTemplate,
                directory: 'classes',
                kind: typedoc_1.ReflectionKind.Class,
            };
        }
        if (isAll || kindsWithOwnFile.includes('interface')) {
            mappings[typedoc_1.ReflectionKind.Interface] = {
                isLeaf: false,
                template: this.reflectionTemplate,
                directory: 'interfaces',
                kind: typedoc_1.ReflectionKind.Interface,
            };
        }
        if (isAll || kindsWithOwnFile.includes('enum')) {
            mappings[typedoc_1.ReflectionKind.Enum] = {
                isLeaf: false,
                template: this.reflectionTemplate,
                directory: 'enums',
                kind: typedoc_1.ReflectionKind.Enum,
            };
        }
        if (isAll || kindsWithOwnFile.includes('function')) {
            mappings[typedoc_1.ReflectionKind.Function] = {
                isLeaf: true,
                template: this.memberTemplate,
                directory: 'functions',
                kind: typedoc_1.ReflectionKind.Function,
            };
        }
        if (isAll || kindsWithOwnFile.includes('typealias')) {
            mappings[typedoc_1.ReflectionKind.TypeAlias] = {
                isLeaf: true,
                template: this.memberTemplate,
                directory: 'types',
                kind: typedoc_1.ReflectionKind.TypeAlias,
            };
        }
        if (isAll || kindsWithOwnFile.includes('variable')) {
            mappings[typedoc_1.ReflectionKind.Variable] = {
                isLeaf: true,
                template: this.memberTemplate,
                directory: 'variables',
                kind: typedoc_1.ReflectionKind.Variable,
            };
        }
        return mappings;
    }
}
exports.MarkdownThemeConverterContext = MarkdownThemeConverterContext;
